unit IGCinterface;

interface

uses Sysutils;

type TLongIntPtr = ^LongInt;
type TCardinalPtr = ^Cardinal;

{$IFDEF VER100}
type Longword = Longint;
type HResult = Longint;
{$ENDIF}

type TLongWordPtr = ^Longword;
type TDoublePtr = ^Double;
type TShortIntPtr = ^ShortInt;
type TIGCresultHandle = Pointer;
type TDateTimePtr = ^TDateTime;
type TBooleanPtr = ^Boolean;
type TIntegerPtr = ^Integer;
type TPCharPtr = ^PChar;

type
   TIGCiunknown = class(TObject)
   protected
      FReferenceCount : Integer;
   public
      constructor Create;
      function _AddRef: Integer; virtual; stdcall;
      function _Release: Integer; virtual; stdcall;

      function QueryInterface(const IID: TGUID; out Obj): HResult; stdcall;
   end;

type
   TIGCexception = class(Exception)
   private
       FLongDescription : string;
       function Get_Message : string;
   public
       function LongDescription : string;
       constructor CreateLong(Description : string);
       property Message : string read Get_Message ;
   end;

procedure DLLcheckResult(ErrorHandle : TIGCresultHandle);

type TIGCexceptionHandle = Pointer;
type TIGCexceptionHandlePtr = ^TIGCexceptionHandle;

type IIGCexception = interface
   {$IFNDEF HIDE}
   {$ENDIF}

   {:
     A text message describing the error.
   }
   function GetDescription : String;

   {:
     An error code representing the error
   }
   function GetCode : Integer;
   end;

function IGCexceptionCreateObject : IIGCexception;
function IGCexceptionCreateObjectWithHandle(Handle : TIGCexceptionHandle) : IIGCexception;

implementation

uses IGC;

type TIGCexceptionImp = class(TIGCiunknown, IIGCexception)
   private
      FHandle : TIGCexceptionHandle;
   public
      constructor Create;
      constructor CreateWithHandle(Handle : TIGCexceptionHandle);
      function _AddRef : Integer; override; stdcall;
      function _Release : Integer; override; stdcall;
      destructor Destroy; override;
      {$IFNDEF HIDE}
      {$ENDIF}

      {:
        A text message describing the error.
      }
      function GetDescription : String;

      {:
        An error code representing the error
      }
      function GetCode : Integer;
   end;

function IGCexceptionCreate
(
   ReturnValue : TIGCexceptionHandlePtr
) : TIGCresultHandle; stdcall;
external 'IGC.dll' name '_IGCexceptionCreate@4';

function IGCexceptionAddRef
(
   Handle : TIGCexceptionHandle
) : TIGCresultHandle; stdcall;
external 'IGC.dll' name '_IGCexceptionAddRef@4';

function IGCexceptionRelease
(
   Handle : TIGCexceptionHandle
) : TIGCresultHandle; stdcall;
external 'IGC.dll' name '_IGCexceptionRelease@4';

function IGCexceptionGetDescription
(
   Handle : TIGCexceptionHandle;
   ReturnValue : TPCharPtr
) : TIGCresultHandle; stdcall;
external 'IGC.dll' name '_IGCexceptionGetDescription@8';

function IGCexceptionGetCode
(
   Handle : TIGCexceptionHandle;
   ReturnValue : TIntegerPtr
) : TIGCresultHandle; stdcall;
external 'IGC.dll' name '_IGCexceptionGetCode@8';

constructor TIGCexceptionImp.Create();
begin
   inherited Create;
   FHandle := nil;
   DLLcheckResult(IGCexceptionCreate(Addr(FHandle)));
end;

constructor TIGCexceptionImp.CreateWithHandle(Handle : TIGCexceptionHandle);
begin
   inherited Create;
   FHandle := Handle;
end;

function TIGCexceptionImp._AddRef : Integer;
begin
   Result := inherited _AddRef;
   DLLcheckResult(IGCexceptionAddRef(FHandle));
end;

function TIGCexceptionImp._Release : Integer;
begin
   DLLcheckResult(IGCexceptionRelease(FHandle));
Result := inherited _Release;
end;

destructor TIGCexceptionImp.Destroy;
begin
   inherited Destroy;
end;

function IGCexceptionCreateObject : IIGCexception;
begin
   Result := TIGCexceptionImp.Create();
end;

function IGCexceptionCreateObjectWithHandle(Handle : TIGCexceptionHandle) : IIGCexception;
begin
   Result := TIGCexceptionImp.CreateWithHandle(Handle);
end;

function TIGCexceptionImp.GetDescription : String;
var
   ResultValue : PChar;
begin
   DLLcheckResult(IGCexceptionGetDescription(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

function TIGCexceptionImp.GetCode : Integer;
var
   ResultValue : Integer;
begin
   DLLcheckResult(IGCexceptionGetCode(FHandle, Addr(ResultValue)));
   Result := ResultValue;
end;

procedure DLLcheckResult(ErrorHandle : TIGCresultHandle);
var
   Description : string;
   Error : IIGCexception;
begin
   if TIGCresultHandle(0) <> ErrorHandle then
   begin
      if ErrorHandle = TIGCresultHandle(-1) then
      begin
         raise TIGCexception.CreateLong('Call failed.');
      end;
      Error := IGCexceptionCreateObjectWithHandle(ErrorHandle);
      Description := Error.GetDescription;
      raise TIGCexception.CreateLong(Description);
   end;
end;

{ TIGCiunknown }

constructor TIGCiunknown.Create;
begin
   FReferenceCount :=0;
end;

function TIGCiunknown.QueryInterface(const IID: TGUID; out Obj): HResult;
begin
   Result := 0;
end;

function TIGCiunknown._AddRef: Integer;
begin
   Inc(FReferenceCount);
   Result := FReferenceCount;
end;

function TIGCiunknown._Release: Integer;
begin
   Dec(FReferenceCount);
   Result := FReferenceCount;
   if (0 = FReferenceCount) then
   begin
      Self.Free;
   end;
end;


{ TIGCexception }

constructor TIGCexception.CreateLong(Description: string);
var
   ShortDescription : String;
begin
   ShortDescription := Description;
   FLongDescription := Description;
   if Length(ShortDescription) > 512 then
   begin
      SetLength(ShortDescription, 512);
   end;
   inherited Create(ShortDescription);
end;

function TIGCexception.Get_Message: string;
begin
   Result := FLongDescription;
end;

function TIGCexception.LongDescription: string;
begin
   Result := FLongDescription;
end;

end.
